;;;; -*- Mode: Lisp -*-
;;;; $Id: usocket-server.lisp 449 2008-09-04 13:16:39Z binghe $

(in-package :usocket)

(defvar *remote-host*)
(defvar *remote-port*)

(defun socket-server (host port function &optional arguments
		      &key (element-type '(unsigned-byte 8)) (timeout 1)
		           (max-buffer-size +max-datagram-packet-size+))
  (let ((socket (socket-connect/udp nil nil
                                    :local-host host
                                    :local-port port
                                    :element-type element-type))
        (buffer (make-array max-buffer-size
                            :element-type '(unsigned-byte 8)
                            :initial-element 0)))
    (unwind-protect
        (loop (progn
		(when (wait-for-input socket :timeout timeout)
		  (multiple-value-bind (recv n *remote-host* *remote-port*)
		      (socket-receive socket buffer max-buffer-size)
		    (declare (ignore recv))
		    (if (plusp n)
			(progn
			  (let ((reply
				 (apply function
					(cons (subseq buffer 0 n) arguments))))
			    (when reply
			      (replace buffer reply)
			      (let ((n (socket-send socket buffer (length reply)
						    :address *remote-host*
						    :port *remote-port*)))
				(when (minusp n)
				  (error "send error: ~A~%" n))))))
			(error "receive error: ~A" n))))
		#+(and cmu mp) (mp:process-yield)))
      (socket-close socket))))
