;;;; -*- Mode: Lisp -*-
;;;; $Id: usocket.lisp 458 2008-09-05 13:54:47Z binghe $

(in-package :usocket)

(export '(stream-datagram-usocket
          socket-connect/udp
          with-udp-client-socket
          socket-send
          socket-receive
          socket-sync
          socket-server
          socket-warning ; condition
          *remote-host*
          *remote-port*))

(defconstant +max-datagram-packet-size+ 65536)

;; redefine datagram-usocket, add rtt-info-mixin
(defclass datagram-usocket (usocket rtt-info-mixin)
  ((connected-p :initarg :connected-p :accessor connected-p))
  (:documentation "UDP (inet-datagram) socket"))

(defclass stream-datagram-usocket (stream-usocket datagram-usocket)
  ()
  (:documentation "connected datagram-usocket with a lisp stream interface"))

(defun stream-datagram-usocket-p (socket)
  (typep socket 'stream-datagram-usocket))

(defun make-stream-datagram-socket (socket stream)
  (unless socket
    (error 'invalid-socket-error))
  (unless stream
    (error 'invalid-socket-stream-error))
  (make-instance 'stream-datagram-usocket
                 :socket socket
                 :stream stream
                 :connected-p t))

(defmacro with-udp-client-socket ((socket-var stream-var &rest socket-connect-args)
                                  &body body)
  "Bind the socket resulting from a call to `socket-connect/udp' with
the arguments `socket-connect-args' to `socket-var' and if `stream-var' is
non-nil, bind the associated socket stream to it."
  `(with-connected-socket (,socket-var (socket-connect/udp ,@socket-connect-args))
     ,(if (null stream-var)
          `(progn ,@body)
        `(let ((,stream-var (socket-stream ,socket-var)))
           ,@body))))
